/***************************************************************************/
/*                                                                         */
/* Datei-Name      : RTC-TEST.C                                            */
/* Project         : RTC-TEST                                              */
/* sonst. Module   : ---                                                   */
/* Compiler        : BC 3.1                                                */
/* Betriebs-System : MS-DOSE                                               */
/* Autor           : J. Kuenstner                                          */
/* Datum           : Jan.  95                                              */
/* Beschreibung    :                                                       */
/*                                                                         */
/*      Entstand aus dem Artikel: "Stechuhr" in c't 11/94 auf Seite 248    */
/*      Hintergrund-Informationen zum RTC-Chip in c't 12/88 auf Seite 196  */
/*      Zugriff auf die Register im RTC erfolgt folgendermassen:           */
/*      Schreiben der Regsister-Nummer auf Port 0x70                       */
/*      Lesen oder Schreiben des gewuenschten Registers auf Port 0x71      */
/*                                                                         */
/*        RTC-Register                                                     */
/*       Ŀ                                       */
/*        0:  Sekunde                                                   */
/*        1:  Alarm-Sekunde                                             */
/*        2:  Minute                                                    */
/*        3:  Alarm-Minute                                              */
/*        4:  Stunde                                                    */
/*        5:  Alarm-Stunde                                              */
/*        6:  Wochentag                                                 */
/*        7:  Tag                                                       */
/*        8:  Monat                                                     */
/*        9:  Jahr                                                      */
/*       10:  StatusRegister A                                          */
/*       11:  StatusRegister B                                          */
/*       12:  StatusRegister C                                          */
/*       13:  StatusRegister D                                          */
/*                                              */
/*                                                                         */
/***************************************************************************/
/*                               HISTORY                                   */
/*                                                                         */
/* Datum     | wer         | was                                           */
/* ------------------------------------------------------------------------*/
/*           |             |                                               */
/* 28.01.95  | Jogi        | Erst-Implementierung                          */
/* ------------------------------------------------------------------------*/
/* 28.02.98  | Jogi        | Kopf reinkopiert wegen "Veroeffentlichung"    */
/*                                                                         */
/***************************************************************************/


#include <stdio.h>
#include <dos.h>
#include <conio.h>
#define UPDATE_MASK 0x80					/* MSB in Register A: Update in Progress */

#define RTC_CHOICE		0x70				/*  Auswahl was gelesen werden soll */
#define RTC_REG			0x71				/*  das eigentliche Schreib-Lese-Register */

#define SEK					0
#define AL_SEK				1
#define MIN					2
#define AL_MIN				3
#define HOUR				4
#define AL_HOUR			5
#define DAY					7
#define MON             8
#define STATUS_A			10


#ifndef __TURBOC__
#define outportb(x,y) outp((x),(y))
#define inportb(x) 	inp(x) 
#endif

typedef unsigned char byte;		/* wir basteln uns ein Byte */
typedef unsigned int  word;		/* wir basteln uns ein Word */

static int bcd2int( int bcd );


int main( void  )
{
	int sek,min,hour,day,mon,oldsek;
	int Amin,Ahour,Aday;
	int test;
	printf("\033[2J");
	printf("Programm RTC-TEST (c) by Jogi 1995-98 !\n");
	printf("Zeigt kontinuierlich die Zeit und die Alarmzeit an.\n");
	printf("So kann man ueberpruefen, ob das TSR: PUTRTC.COM aktiv ist .\n");
	printf("Achtung ! Sekunden von Alarm sind fuer Tag missbraucht !\n");
	printf("\nEnde: beliebige Taste druecken  \n\n");
	do
	{
		_disable();   										/* Interrupt sperren */
		outportb( RTC_CHOICE, STATUS_A );			/* Zugriff auf Register A */
		test = (inportb( RTC_REG ) & UPDATE_MASK );
		_enable();											/* und wieder zulassen */
		if(!test)											/* wenn Bit 7 = 0 dann Gut */
		{
			_disable();
			outportb( RTC_CHOICE, SEK );
			sek = bcd2int(inportb( RTC_REG ));
			outportb( RTC_CHOICE, MIN );
			min = bcd2int(inportb( RTC_REG ));
			outportb( RTC_CHOICE, HOUR );
			hour = bcd2int(inportb( RTC_REG ));
			outportb( RTC_CHOICE, DAY );
			day = bcd2int(inportb( RTC_REG ));
			outportb( RTC_CHOICE, MON );
			mon = bcd2int(inportb( RTC_REG ));


			_enable();

			_disable();
			outportb( RTC_CHOICE, AL_SEK );
			Aday = bcd2int(inportb( RTC_REG ));
			outportb( RTC_CHOICE, AL_MIN );
			Amin = bcd2int(inportb( RTC_REG ));
			outportb( RTC_CHOICE, AL_HOUR );
			Ahour = bcd2int(inportb( RTC_REG ));
			_enable();
			if(sek != oldsek)
			{
				oldsek = sek;
				printf("\033[10;0H");
				printf(" ZEIT : %02d.%02d   %02d:%02d:%02d\n",day,mon,hour,min,sek);
				printf(" ALARM: %02d.??   %02d:%02d:??\n",Aday,Ahour,Amin);
			}
		}
	}
	while(!kbhit());
	printf( "\n\n\n");
	printf( "Kommentare, Probleme, ... an:  jogi@compuserve.com\n");
	printf( "Homepage: http://ourworld.compuserve.com/homepages/jogi\n");
	return 0;
}

static int bcd2int( int bcd )
{
	return (int) (( ((bcd >> 4) & 0xF ) * 10 ) + ( bcd & 0xF ));
}


